@@ -202,10 +202,9 @@ class PsmPy:
                 predicted_data_reset, treatment_dfonly, how='inner', on=self.indx)
             predicted_data_control = self.predicted_data[self.predicted_data[self.treatment] == 0]
             predicted_data_treatment = self.predicted_data[self.predicted_data[self.treatment] == 1]
-
             # return predicted_data
 
-    def knn_matched(self, matcher, replacement=False, caliper=None):
+    def knn_matched(self, matcher='propensity_logit', replacement=False, caliper=None):
         """
         knn_matched -- Match data using k-nn algorithm
         Parameters
@@ -246,108 +245,108 @@ class PsmPy:
         knn = NearestNeighbors(n_neighbors=len(major_pred_rstindx), p=2)
         knn.fit(major_pred_rstindx[[matcher]].to_numpy())
         distances, indexes = knn.kneighbors(
-            minor_pred_rstindx[[matcher]], n_neighbors=len(major_pred_rstindx))
-        self.distances = distances
-        self.indexes = indexes
+            minor_pred_rstindx[[matcher]].to_numpy(), n_neighbors=len(major_pred_rstindx))
+#         self.distances = distances
+#         self.indexes = indexes
 
         def condition_caliper(x, caliper):
             return x <= caliper
 
         # remove elements outside of radius:
         if caliper is not None:
-            self.indexes_for_match = []
-            self.elements_to_remove = []
+            indices_for_match = []
+            elements_to_remove = []
             # loop through both distance and indexes from knn simultaneously:
             for dist, row in zip(distances[:, :], indexes[:, :]):
                 # convert dist and row_ids to lists:
                 dist = np.ndarray.tolist(dist)
                 row = np.ndarray.tolist(row)
                 # finds indices to include from distances based on caliper:
-                self.dist_indices = [idx for idx, element in enumerate(
+                dist_indices = [idx for idx, element in enumerate(
                     dist) if condition_caliper(element, caliper)]
                 # clean up the ids from the distance exclusion:
-                self.row_clean = [row[index] for index in self.dist_indices]
+                row_clean = [row[index] for index in dist_indices]
                 # check to see if replacement is False:
                 if replacement == False:
-                    if len(self.elements_to_remove) > 0:
-                        for element in self.elements_to_remove:
+                    if len(elements_to_remove) > 0:
+                        for element in elements_to_remove:
                             # check is element is in
-                            if element in self.row_clean:
-                                self.row_clean.remove(element)
+                            if element in row_clean:
+                                row_clean.remove(element)
                             else:
                                 pass
                         try:
-                            self.indexes_for_match.append(self.row_clean[0])
-                            self.elements_to_remove.append(self.row_clean[0])
+                            indices_for_match.append(row_clean[0])
+                            elements_to_remove.append(row_clean[0])
                         except:
                             # append None if nothing within caliper range that also isn't a duplicate
-                            self.indexes_for_match.append(None)
+                            indices_for_match.append(None)
 
                     else:
                         try:
-                            self.indexes_for_match.append(self.row_clean[0])
-                            self.elements_to_remove.append(self.row_clean[0])
+                            indices_for_match.append(row_clean[0])
+                            elements_to_remove.append(row_clean[0])
                         except:
-                            self.indexes_for_match.append(None)
+                            indices_for_match.append(None)
 
                 # if replacement is True:
                 else:
                     # try and append from list (if caliper size ok)
                     try:
-                        self.indexes_for_match.append(self.row_clean[0])
+                        indices_for_match.append(row_clean[0])
                     # otherwise append None
                     except:
-                        self.indexes_for_match.append(None)
+                        indices_for_match.append(None)
 
         # if no caliper:
         else:
-            self.indexes_for_match = []
-            self.elements_to_remove = []
+            indices_for_match = []
+            elements_to_remove = []
             # loop through both distance and indexes from knn simultaneously:
             for dist, row in zip(distances[:, :], indexes[:, :]):
                 # convert dist and row_ids to lists:
                 dist = np.ndarray.tolist(dist)
                 row = np.ndarray.tolist(row)
-                self.row_clean = row
+                row_clean = row
                 # check to see if replacement is False:
                 if replacement == False:
                     # if there are elements in elements to remove:
-                    if len(self.elements_to_remove) > 0:
-                        for element in self.elements_to_remove:
+                    if len(elements_to_remove) > 0:
+                        for element in elements_to_remove:
                             # check is element is in row clean
-                            if element in self.row_clean:
-                                self.row_clean.remove(element)
+                            if element in row_clean:
+                                row_clean.remove(element)
                             else:
                                 pass
                         try:
-                            self.indexes_for_match.append(self.row_clean[0])
-                            self.elements_to_remove.append(self.row_clean[0])
+                            indices_for_match.append(row_clean[0])
+                            elements_to_remove.append(row_clean[0])
                         except:
                             # append None if nothing within caliper range that also isn't a duplicate
-                            self.indexes_for_match.append(None)
+                            indices_for_match.append(None)
                     # if there are NO elements in elements to remove:
                     else:
                         try:
-                            self.indexes_for_match.append(self.row_clean[0])
-                            self.elements_to_remove.append(self.row_clean[0])
+                            indices_for_match.append(row_clean[0])
+                            elements_to_remove.append(row_clean[0])
                         except:
-                            self.indexes_for_match.append(None)
+                            indices_for_match.append(None)
                 # if replacement is True:
                 else:
                     # try and append from list (if caliper size ok)
                     try:
-                        self.indexes_for_match.append(self.row_clean[0])
+                        indices_for_match.append(row_clean[0])
                     # otherwise append None
                     except:
-                        self.indexes_for_match.append(None)
+                        indices_for_match.append(None)
 
-        if len(indexes[:, 1]) == len(np.unique(list(filter(None, self.indexes_for_match)))):
+        if len(indexes[:, 1]) == len(np.unique(list(filter(None, indices_for_match)))):
             pass
         else:
             warnings.warn('Some values do not have a match')
 
         ID_match = []
-        for idxxx in self.indexes_for_match:
+        for idxxx in indices_for_match:
             # try to pull original patient indexes specified at outset
             try:
                 match = major_pred_rstindx.loc[idxxx, self.indx]
@@ -356,16 +355,16 @@ class PsmPy:
             except:
                 ID_match.append(np.nan)
 
-        indexes_nonull = list(filter(None, self.indexes_for_match))
+        indexes_nonull = list(filter(None, indices_for_match))
         major_matched = major_pred_rstindx.take(indexes_nonull)
         self.df_matched = pd.concat(
             [minor_pred_rstindx, major_matched], axis=0, ignore_index=True)
         minor_pred_rstindx['matched_ID'] = ID_match
         self.matched_ids = minor_pred_rstindx[[self.indx, 'matched_ID']]
 
-    def knn_matched_12n(self, matcher, how_many=1):
+    def knn_matched_12n(self, matcher='propensity_logit', how_many=1):
         """
-        knn_matched -- Match data using k-nn algorithm
+        knn_matched_12n -- Match data using k-nn algorithm to sample 1:selected_number
         Parameters
         ----------
         matcher : str
@@ -396,26 +395,30 @@ class PsmPy:
             minor_pred_rstindx = min_pred.reset_index(drop=True)
         major_pred_rstindx_og = major_pred_rstindx.copy()
 
+        #self.indices_for_match = []
+        ID_match = []
         # indexes for match needs to keep all those tagged from all loops so must be instantiated outside the loop
-        self.indexes_for_match = []
-
         for many in range(how_many):
             # need to fit KNN with larger class
             knn = NearestNeighbors(n_neighbors=len(major_pred_rstindx), p=2)
             knn.fit(major_pred_rstindx[[matcher]].to_numpy())
             distances, indexes = knn.kneighbors(
-                minor_pred_rstindx[[matcher]], n_neighbors=len(major_pred_rstindx))
-            # self.distances = distances
-            # self.indexes = indexes
+                minor_pred_rstindx[[matcher]].to_numpy(), n_neighbors=len(major_pred_rstindx))
+#             self.distances = distances
+#             self.indexes = indexes
 
             # lists to remove elements that have already matched
             elements_to_remove = []
+            indices_in_loop = []
             # loop through both distance and indexes from knn simultaneously:
             for dist, row in zip(distances[:, :], indexes[:, :]):
                 # convert dist and row_ids to lists:
                 dist = np.ndarray.tolist(dist)
                 row = np.ndarray.tolist(row)
-                # check to see if you need to remove elements:
+#                 self.dist_indices = [idx for idx, element in enumerate(
+#                     dist)]
+                # clean up the ids from the distance exclusion:
+                #self.row_clean = row
                 if len(elements_to_remove) > 0:
                     for element in elements_to_remove:
                         # check is element is in
@@ -423,39 +426,44 @@ class PsmPy:
                             row.remove(element)
                         else:
                             pass
-                        self.indexes_for_match.append(row[0])
-                        elements_to_remove.append(row[0])
+                    elements_to_remove.append(row[0])
+                    indices_in_loop.append(row[0])
                 # if there are NO elements in elements to remove:
                 else:
-                    self.indexes_for_match.append(row[0])
                     elements_to_remove.append(row[0])
+                    indices_in_loop.append(row[0])
 
             # remove all used indexes from this round of KNN and prepare for next with the last set removed:
             # create df for exclusion on those that have matched
-            major_matched = major_pred_rstindx.take(self.indexes_for_match)
-            # run the exclusion for the next run:
-            major_pred_rstindx = pd.merge(major_pred_rstindx, major_matched, on='matched_ID', how="outer", indicator=True
-                                          ).query('_merge=="left_only"')
+            major_matched_loop = major_pred_rstindx.take(indices_in_loop)
+            # pull patient indexes (self.indx) from dataframe as we go (since array values will repeat):
+            for idxxx in indices_in_loop:
+                match = major_pred_rstindx.loc[idxxx, self.indx]
+                ID_match.append(match)
 
-        # initiate string for ID that will grab the row in accordance with the indx
-        ID_match = []
-        for idxxx in self.indexes_for_match:
-            match = major_pred_rstindx_og.loc[idxxx, self.indx]
-            ID_match.append(match)
-            ID_match_array = np.asarray(ID_match).reshape(
-                (len(minor_pred_rstindx), many))
+#             # run the exclusion for the next run:
+            major_pred_rstindx = pd.merge(major_pred_rstindx, major_matched_loop[[
+                                          self.indx]], on=self.indx, how="outer", indicator=True).query('_merge=="left_only"')
+            major_pred_rstindx = major_pred_rstindx.drop(['_merge'], axis=1)
+            # reset index for next round of matching
+            major_pred_rstindx = major_pred_rstindx.reset_index(drop=True)
 
         # build dataframe passed back to user
-        major_matched = major_pred_rstindx_og.take(self.indexes_for_match)
+        df_2_extract_major_class = pd.DataFrame(ID_match, columns=[self.indx])
+        df_4_user = pd.merge(
+            major_pred_rstindx_og, df_2_extract_major_class, on=self.indx, how="right")
         self.df_matched = pd.concat(
-            [minor_pred_rstindx, major_matched], axis=0, ignore_index=True)
+            [minor_pred_rstindx, df_4_user], axis=0, ignore_index=True)
 
+        # build df of matched ids passed back to user:
+        # create array for easy viewing of the 'n' levels of macthing performed
+        ID_match_array = np.asarray(ID_match).reshape(
+            (len(minor_pred_rstindx), how_many))
         major_list_cols = []
-        # build df of matched ids passed back to user
         for many in range(how_many):
-            col_name = 'largerclass_id' + str(many)
+            col_name = 'largerclass_' + str(many) + 'group'
             major_list_cols.append(col_name)
-
+        # convert matched ids from major class to df
         major_class_matched_ids_df = pd.DataFrame(
             ID_match_array, columns=major_list_cols)
         self.matched_ids = pd.concat(
